
class Portfolio {

    constructor(portfolio) {
        this.positions = portfolio.positions
        this.total = portfolio.total
        this.prices = this.positions.reduce((a, c) => { a[c.instrument] = c.price; return a }, {})
        this.update()
    }

    setPrice(instrument, price) {
        this.prices[instrument] = price
        this.update()
    }

    order(instrument, quantity) {
        const value = this.prices[instrument] * quantity
        this.changeCash(-value)

        const position = this.getPosition(instrument)
        position.quantity += quantity
        this.update()
    }

    update() {
        this.positions.forEach(p => p.price = this.prices[p.instrument])
        this.positions.forEach(p => p.value = p.quantity * p.price)
        this.total = this.positions.reduce((a, c) => a + c.value, 0)
    }

    changeCash(amount) {
        const USDPosition = this.positions.find(p => p.instrument === "USD")
        USDPosition.quantity += amount
    }

    getPosition(instrument){
        let position = this.positions.find(p => p.instrument === instrument)
        if (position == null) {
            position = { instrument, price: 0, quantity: 0, value: 0 }
        }
        return position
    }

    toObject() {
        return {
            positions: this.positions,
            total: this.total
        }
    }
}

module.exports = Portfolio