const Portfolio = require('./portfolio')
const sell_strategy = require('./sell_strategy')

class Simulator {

    constructor() {
        this.moment = null
    }

    init(moment) {
        this.moment = moment
    }

    simulate(times, prices, predictions, sellStrategy, allowMultiTrade, dataKind) {
        let result = []
        let profitLoss = 0
        let trades = []
        for (const [i, price] of prices.entries()) {

            const time = times[i]
            const prediction = predictions[i]

            // sell
            for (let trade of trades) {
                const sellStrategies = trade.sellStrategy.split('|')
                for (let sellStrategy of sellStrategies) {
                    sell_strategy[sellStrategy](this.moment, time, price, trade)
                    if (trade.status == "closed") break;
                }
            }

            // realize profits on closed trades
            trades.filter(p => p.status == "closed").forEach(p => profitLoss += p.profitLoss)

            // remove closed trades
            trades = trades.filter(p => p.status != "closed")

            // buy
            const buyEnabled = allowMultiTrade || trades.length == 0
            if (buyEnabled) {

                const buyRate = prediction
                if (buyRate > 0) {

                    const position = buyRate //buyRate / price

                    const trade = {
                        type: "long",
                        status: "open",
                        profitLoss: 0,
                        time,
                        price,
                        position,
                        sellStrategy
                    }

                    trades.push(trade)
                }
            }

            if (dataKind == 'tradesCount') {
                result.push(trades.length)
            }
            else if (dataKind == 'profitLoss') {
                result.push(profitLoss)
            }
        }

        return result
    }
}

module.exports = new Simulator()